/*  This file is part of the program psim.

    Copyright (C) 1994-1996, Andrew Cagney <cagney@highland.com.au>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    */


#ifndef _HW_PHB_H_
#define _HW_PHB_H_

typedef enum {
  hw_phb_memory_space,
  hw_phb_io_space,
  hw_phb_config_space,
  hw_phb_special_space,
  nr_hw_phb_spaces,
} hw_phb_spaces;

typedef enum  {
  hw_phb_normal_decode = attach_callback,
  hw_phb_subtractive_decode,
  hw_phb_master_abort_decode,
} hw_phb_decode;


#endif /* _HW_PHB_H_ */
