/* Version information for GDB.
   Copyright (C) 1999, 2007 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#ifndef VERSION_H
#define VERSION_H

/* Version number of GDB, as a string.  */
extern const char version[];

/* Canonical host name as a string. */
extern const char host_name[];

/* Canonical target name as a string. */
extern const char target_name[];

#endif /* #ifndef VERSION_H */
